@@ -141,7 +141,14 @@ glnx_renameat2_exchange (int olddirfd, const char *oldpath,
 #endif
 
   /* Fallback */
-  { const char *old_tmp_name = glnx_strjoina (oldpath, ".XXXXXX");
+  { char *old_tmp_name_buf = glnx_strjoina (oldpath, ".XXXXXX");
+    /* This obviously isn't race-free, but doing better gets tricky, since if
+     * we're here the kernel isn't likely to support RENAME_NOREPLACE either.
+     * Anyways, upgrade the kernel. Failing that, avoid use of this function in
+     * shared subdirectories like /tmp.
+     */
+    glnx_gen_temp_name (old_tmp_name_buf);
+    const char *old_tmp_name = old_tmp_name_buf;
 
     /* Move old out of the way */
     if (renameat (olddirfd, oldpath, olddirfd, old_tmp_name) < 0)
@@ -168,11 +175,7 @@ glnx_tmpfile_clear (GLnxTmpfile *tmpf)
     return;
   if (!tmpf->initialized)
     return;
-  if (tmpf->fd != -1)
-    {
-      if (close (tmpf->fd) < 0)
-        g_assert (errno != EBADF);
-    }
+  glnx_close_fd (&tmpf->fd);
   /* If ->path is set, we're likely aborting due to an error. Clean it up */
   if (tmpf->path)
     {
@@ -188,9 +191,8 @@ open_tmpfile_core (int dfd, const char *subpath,
                    GLnxTmpfile *out_tmpf,
                    GError **error)
 {
+  /* Picked this to match mkstemp() */
   const guint mode = 0600;
-  glnx_fd_close int fd = -1;
-  int count;
 
   dfd = glnx_dirfd_canonicalize (dfd);
 
@@ -201,33 +203,35 @@ open_tmpfile_core (int dfd, const char *subpath,
    * link_tmpfile() below to rename the result after writing the file
    * in full. */
 #if defined(O_TMPFILE) && !defined(DISABLE_OTMPFILE) && !defined(ENABLE_WRPSEUDO_COMPAT)
-  fd = openat (dfd, subpath, O_TMPFILE|flags, mode);
-  if (fd == -1 && !(G_IN_SET(errno, ENOSYS, EISDIR, EOPNOTSUPP)))
-    return glnx_throw_errno_prefix (error, "open(O_TMPFILE)");
-  if (fd != -1)
-    {
-      /* Workaround for https://sourceware.org/bugzilla/show_bug.cgi?id=17523
-       * See also https://github.com/ostreedev/ostree/issues/991
-       */
-      if (fchmod (fd, mode) < 0)
-        return glnx_throw_errno_prefix (error, "fchmod");
-      out_tmpf->initialized = TRUE;
-      out_tmpf->src_dfd = dfd; /* Copied; caller must keep open */
-      out_tmpf->fd = glnx_steal_fd (&fd);
-      out_tmpf->path = NULL;
-      return TRUE;
-    }
+  {
+    glnx_autofd int fd = openat (dfd, subpath, O_TMPFILE|flags, mode);
+    if (fd == -1 && !(G_IN_SET(errno, ENOSYS, EISDIR, EOPNOTSUPP)))
+      return glnx_throw_errno_prefix (error, "open(O_TMPFILE)");
+    if (fd != -1)
+      {
+        /* Workaround for https://sourceware.org/bugzilla/show_bug.cgi?id=17523
+         * See also https://github.com/ostreedev/ostree/issues/991
+         */
+        if (fchmod (fd, mode) < 0)
+          return glnx_throw_errno_prefix (error, "fchmod");
+        out_tmpf->initialized = TRUE;
+        out_tmpf->src_dfd = dfd; /* Copied; caller must keep open */
+        out_tmpf->fd = glnx_steal_fd (&fd);
+        out_tmpf->path = NULL;
+        return TRUE;
+      }
+  }
   /* Fallthrough */
 #endif
 
+  const guint count_max = 100;
   { g_autofree char *tmp = g_strconcat (subpath, "/tmp.XXXXXX", NULL);
-    const guint count_max = 100;
 
-    for (count = 0; count < count_max; count++)
+    for (int count = 0; count < count_max; count++)
       {
         glnx_gen_temp_name (tmp);
 
-        fd = openat (dfd, tmp, O_CREAT|O_EXCL|O_NOFOLLOW|O_NOCTTY|flags, mode);
+        glnx_autofd int fd = openat (dfd, tmp, O_CREAT|O_EXCL|O_NOFOLLOW|O_NOCTTY|flags, mode);
         if (fd < 0)
           {
             if (errno == EEXIST)
@@ -246,7 +250,7 @@ open_tmpfile_core (int dfd, const char *subpath,
       }
   }
   g_set_error (error, G_IO_ERROR, G_IO_ERROR_EXISTS,
-               "Exhausted %u attempts to create temporary file", count);
+               "Exhausted %u attempts to create temporary file", count_max);
   return FALSE;
 }
 
@@ -356,9 +360,9 @@ glnx_link_tmpfile_at (GLnxTmpfile *tmpf,
           char *dnbuf = strdupa (target);
           const char *dn = dirname (dnbuf);
           char *tmpname_buf = glnx_strjoina (dn, "/tmp.XXXXXX");
-          guint count;
-          const guint count_max = 100;
 
+          const guint count_max = 100;
+          guint count;
           for (count = 0; count < count_max; count++)
             {
               glnx_gen_temp_name (tmpname_buf);
@@ -569,7 +573,7 @@ glnx_file_get_contents_utf8_at (int                   dfd,
 {
   dfd = glnx_dirfd_canonicalize (dfd);
 
-  glnx_fd_close int fd = -1;
+  glnx_autofd int fd = -1;
   if (!glnx_openat_rdonly (dfd, subpath, TRUE, &fd, error))
     return NULL;
 
@@ -599,17 +603,13 @@ glnx_readlinkat_malloc (int            dfd,
                         GCancellable  *cancellable,
                         GError       **error)
 {
-  size_t l = 100;
-
   dfd = glnx_dirfd_canonicalize (dfd);
 
+  size_t l = 100;
   for (;;)
     {
-      g_autofree char *c = NULL;
-      ssize_t n;
-
-      c = g_malloc (l);
-      n = TEMP_FAILURE_RETRY (readlinkat (dfd, subpath, c, l-1));
+      g_autofree char *c = g_malloc (l);
+      ssize_t n = TEMP_FAILURE_RETRY (readlinkat (dfd, subpath, c, l-1));
       if (n < 0)
         return glnx_null_throw_errno_prefix (error, "readlinkat");
 
@@ -679,18 +679,15 @@ copy_symlink_at (int                   src_dfd,
 int
 glnx_loop_write(int fd, const void *buf, size_t nbytes)
 {
-  const uint8_t *p = buf;
-
-  g_return_val_if_fail(fd >= 0, -1);
-  g_return_val_if_fail(buf, -1);
+  g_return_val_if_fail (fd >= 0, -1);
+  g_return_val_if_fail (buf, -1);
 
   errno = 0;
 
+  const uint8_t *p = buf;
   while (nbytes > 0)
     {
-      ssize_t k;
-
-      k = write(fd, p, nbytes);
+      ssize_t k = write(fd, p, nbytes);
       if (k < 0)
         {
           if (errno == EINTR)
@@ -926,7 +923,7 @@ glnx_file_copy_at (int                   src_dfd,
 
   /* Regular file path below here */
 
-  glnx_fd_close int src_fd = -1;
+  glnx_autofd int src_fd = -1;
   if (!glnx_openat_rdonly (src_dfd, src_subpath, FALSE, &src_fd, error))
     return FALSE;
 
