@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2015 Colin Walters <walters@verbum.org>
  *
+ * SPDX-License-Identifier: LGPL-2.0+
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -200,7 +202,7 @@ out:
 }
 
 gboolean
-ostree_builtin_gpg_sign (int argc, char **argv, GCancellable *cancellable, GError **error)
+ostree_builtin_gpg_sign (int argc, char **argv,OstreeCommandInvocation *invocation, GCancellable *cancellable, GError **error)
 {
   g_autoptr(GOptionContext) context = NULL;
   g_autoptr(OstreeRepo) repo = NULL;
@@ -210,9 +212,9 @@ ostree_builtin_gpg_sign (int argc, char **argv, GCancellable *cancellable, GErro
   int n_key_ids, ii;
   gboolean ret = FALSE;
 
-  context = g_option_context_new ("COMMIT KEY-ID... - Sign a commit");
+  context = g_option_context_new ("COMMIT KEY-ID...");
 
-  if (!ostree_option_context_parse (context, options, &argc, &argv, OSTREE_BUILTIN_FLAG_NONE, &repo, cancellable, error))
+  if (!ostree_option_context_parse (context, options, &argc, &argv, invocation, &repo, cancellable, error))
     goto out;
 
   if (argc < 2)
