@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2011 Colin Walters <walters@verbum.org>
  *
+ * SPDX-License-Identifier: LGPL-2.0+
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -64,7 +66,11 @@ ostree_option_context_new_with_commands (OstreeCommand *commands)
 
   while (commands->name != NULL)
     {
-      g_string_append_printf (summary, "\n  %s", commands->name);
+      g_string_append_printf (summary, "\n  %-18s", commands->name);
+
+      if (commands->description != NULL )
+        g_string_append_printf (summary, "%s", commands->description);
+
       commands++;
     }
 
@@ -163,13 +169,11 @@ ostree_run (int    argc,
 
   if (!command->fn)
     {
-      g_autoptr(GOptionContext) context = NULL;
-      g_autofree char *help = NULL;
-
-      context = ostree_option_context_new_with_commands (commands);
+      g_autoptr(GOptionContext) context =
+        ostree_option_context_new_with_commands (commands);
 
       /* This will not return for some options (e.g. --version). */
-      if (ostree_option_context_parse (context, NULL, &argc, &argv, OSTREE_BUILTIN_FLAG_NO_REPO, NULL, cancellable, &error))
+      if (ostree_option_context_parse (context, NULL, &argc, &argv, NULL, NULL, cancellable, &error))
         {
           if (command_name == NULL)
             {
@@ -191,8 +195,8 @@ ostree_run (int    argc,
   prgname = g_strdup_printf ("%s %s", g_get_prgname (), command_name);
   g_set_prgname (prgname);
 #endif
-
-  if (!command->fn (argc, argv, cancellable, &error))
+  OstreeCommandInvocation invocation = { .command = command };
+  if (!command->fn (argc, argv, &invocation, cancellable, &error))
     goto out;
 
   success = TRUE;
@@ -294,13 +298,39 @@ ostree_option_context_parse (GOptionContext *context,
                              const GOptionEntry *main_entries,
                              int *argc,
                              char ***argv,
-                             OstreeBuiltinFlags flags,
+                             OstreeCommandInvocation *invocation,
                              OstreeRepo **out_repo,
                              GCancellable *cancellable,
                              GError **error)
 {
   g_autoptr(OstreeRepo) repo = NULL;
+  /* When invocation is NULL,  do not fetch repo */
+  const OstreeBuiltinFlags flags = invocation ? invocation->command->flags : OSTREE_BUILTIN_FLAG_NO_REPO;
 
+  if (invocation && invocation->command->description != NULL)
+    {
+      const char *context_summary = g_option_context_get_summary (context);
+
+      /* If the summary is originally empty, we set the description, but
+       * for root commands(command with subcommands), we want to prepend
+       * the description to the existing summary string
+       */
+      if (context_summary == NULL)
+        g_option_context_set_summary (context, invocation->command->description);
+      else
+        {
+          /* TODO: remove this part once we deduplicate the ostree_option_context_new_with_commands
+           * function from other root commands( command with subcommands). Because
+           * we can directly add the summary inside the ostree_option_context_new_with_commands function.
+           */
+          g_autoptr(GString) new_summary_string = g_string_new (context_summary);
+
+          g_string_prepend (new_summary_string, "\n\n");
+          g_string_prepend (new_summary_string, invocation->command->description);
+
+          g_option_context_set_summary (context, new_summary_string->str);
+        }
+    }
   /* Entries are listed in --help output in the order added.  We add the
    * main entries ourselves so that we can add the --repo entry first. */
 
@@ -364,6 +394,7 @@ ostree_admin_option_context_parse (GOptionContext *context,
                                    int *argc,
                                    char ***argv,
                                    OstreeAdminBuiltinFlags flags,
+                                   OstreeCommandInvocation *invocation,
                                    OstreeSysroot **out_sysroot,
                                    GCancellable *cancellable,
                                    GError **error)
@@ -374,7 +405,7 @@ ostree_admin_option_context_parse (GOptionContext *context,
   g_option_context_add_main_entries (context, global_admin_entries, NULL);
 
   if (!ostree_option_context_parse (context, main_entries, argc, argv,
-                                    OSTREE_BUILTIN_FLAG_NO_REPO, NULL, cancellable, error))
+                                    invocation, NULL, cancellable, error))
     return FALSE;
 
   if (!opt_print_current_dir && (flags & OSTREE_ADMIN_BUILTIN_FLAG_NO_SYSROOT))
